<?php

namespace Teknisa\FAI\Modules\Apuracao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;

class FAI04200VendaAntecipadaPorCliente extends \Zeedhi\Framework\Controller\Report {

    protected $birtStrategy;
    protected $environment;
    protected $crudService;
    protected $novoCodigo;
    protected $entityManager;

    public function __construct(ReportService $reportService, $birtStrategy, $environment, $crudService, $novoCodigo, $entityManager) {
        $this->birtStrategy  = $birtStrategy;
        $this->environment  = $environment;
        $this->crudService = $crudService;
        $this->novoCodigo = $novoCodigo;
        $this->entityManager = $entityManager;
        parent::__construct($reportService);
    }
    protected $reportMapping = array (
        'FAI42000' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDCLIENTE' => 'P_CDCLIENTE',
                'NRLISTASERV' => 'P_NRLISTASERV',
                'DTINICIAL' => 'P_DTINICIO',
                'DTFINAL' => 'P_DTFIN',
                'NRORG'  => 'P_NRORG'
            ),
        ),
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row["CDCLIENTE"] = $row["ALL_CDCLIENTE"]? "T": implode("_", $row["CDCLIENTE"]);
        $row["NRLISTASERV"] = $this->geraLista($row);
        $row["NRORG"] = $this->environment->getNrorg();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
    }

    public function geraLista($row){
        $nrorg = $this->environment->getNrorg();
        $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $nrorg);
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";
        
        $filiserv = $row["ALL_CDSERVFILI"]? $this->infoServico($row): $row["FILISERV"];
        foreach ($filiserv as $servicoFilial) {
            if(!empty($servicoFilial)){
                $stmt = $this->entityManager->getConnection()->prepare($sql);
                $stmt->execute(
                    array(
                        "NRLISTA" => $nrlista,
                        "CDLISTA" => $servicoFilial,
                        "NRORG" => $nrorg
                    )
                );
            }
        }
        
        return $nrlista;
    }

    private function infoServico($row){
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator"   => '=',
                "value"      => $row['CDFILIAL']
            ),
            array(
                "columnName" => "CDCLIENTE",
                "operator"   => '=',
                "value"      => $row['CDCLIENTE']
            )
        );
         
        $dataSet = $this->crudService->getFilteredDataSet("fai04200_servico_cliente_filial", $conditions);
        $rows = $dataSet->getRows();
        $filiserv = [];
        foreach($rows as $row){
            array_push($filiserv, $row['FILISERV']);
        }
        return $filiserv;
    }

}