<?php
namespace Teknisa\FAI\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Fai01300FaturamentoDeRefeicoesPorPeriodo extends \Zeedhi\Framework\Controller\Report {

    protected $connection;
    protected $environment;
    protected $newCode;
    protected $crudService;
    protected $qr2Strategy;
    protected $birtStrategy;

    public function __construct($reportService, $connection, $environment, $newCode, $crudService, $birtStrategy, $qr2Strategy) {
        $this->connection               = $connection;
        $this->environment              = $environment;
        $this->newCode                  = $newCode;
        $this->crudService              = $crudService;
        $this->birtStrategy             = $birtStrategy;
        $this->qr2Strategy              = $qr2Strategy;
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'FAI01300' =>
        array (
            'strategy'          => 'BIRT',
            'parameterMapping'  =>
            array (
                'CDFILIAL'     => 'CDFILIAL',
                'NRAPURACGD'   => 'NRAPURACGD',
                'CDCLIENTE'    => 'CDCLIENTE',
                'INICIAL'      => 'DTINICIAL',
                'FINAL'        => 'DTFINAL',
                'QUEBRAFILIAL' => 'IDQUEBRAFILIAL',
                'NRORG'        => 'NRORG'
            )
        ),
        'FAI01301' =>
        array (
            'strategy'          => 'BIRT',
            'parameterMapping'  =>
            array (
                'CDFILIAL'     => 'CDFILIAL',
                'NRAPURACGD'   => 'NRAPURACGD',
                'CDCLIENTE'    => 'CDCLIENTE',
                'INICIAL'      => 'DTINICIAL',
                'FINAL'        => 'DTFINAL',
                'QUEBRAFILIAL' => 'IDQUEBRAFILIAL',
                'NRORG'        => 'NRORG'
            )
        ),
        'FAI01302' =>
        array (
            'strategy'          => 'BIRT',
            'parameterMapping'  =>
            array (
                'CDFILIAL'     => 'CDFILIAL',
                'NRAPURACGD'   => 'NRAPURACGD',
                'CDCLIENTE'    => 'CDCLIENTE',
                'INICIAL'      => 'DTINICIAL',
                'FINAL'        => 'DTFINAL',
                'QUEBRAFILIAL' => 'IDQUEBRAFILIAL',
                'NRORG'        => 'NRORG'
            )
        ),
        'FAI01303' =>
        array (
            'strategy'          => 'BIRT',
            'parameterMapping'  =>
            array (
                'CDFILIAL'     => 'CDFILIAL',
                'NRAPURACGD'   => 'NRAPURACGD',
                'CDCLIENTE'    => 'CDCLIENTE',
                'INICIAL'      => 'DTINICIAL',
                'FINAL'        => 'DTFINAL',
                'QUEBRAFILIAL' => 'IDQUEBRAFILIAL',
                'NRORG'        => 'NRORG'
            )
        ),
        'FAI01304' =>
        array (
            'strategy'          => 'BIRT',
            'parameterMapping'  =>
            array (
                'CDFILIAL'     => 'CDFILIAL',
                'NRAPURACGD'   => 'NRAPURACGD',
                'CDCLIENTE'    => 'CDCLIENTE',
                'INICIAL'      => 'DTINICIAL',
                'FINAL'        => 'DTFINAL',
                'QUEBRAFILIAL' => 'IDQUEBRAFILIAL',
                'NRORG'        => 'NRORG'
            )
        ),
        'FAI01305' =>
        array (
            'strategy'          => 'BIRT',
            'parameterMapping'  =>
            array (
                'CDFILIAL'     => 'CDFILIAL',
                'NRAPURACGD'   => 'NRAPURACGD',
                'CDCLIENTE'    => 'CDCLIENTE',
                'INICIAL'      => 'DTINICIAL',
                'FINAL'        => 'DTFINAL',
                'QUEBRAFILIAL' => 'IDQUEBRAFILIAL',
                'NRORG'        => 'NRORG'
            )
        ),
        'FAI01306' =>
        array (
            'strategy'          => 'BIRT',
            'parameterMapping'  =>
            array (
                'CDFILIAL'     => 'CDFILIAL',
                'NRAPURACGD'   => 'NRAPURACGD',
                'CDCLIENTE'    => 'CDCLIENTE',
                'INICIAL'      => 'DTINICIAL',
                'FINAL'        => 'DTFINAL',
                'QUEBRAFILIAL' => 'IDQUEBRAFILIAL',
                'NRORG'        => 'NRORG'
            )
        ),
        'FAI01307' =>
        array (
            'strategy'          => 'BIRT',
            'parameterMapping'  =>
            array (
                'CDFILIAL'     => 'CDFILIAL',
                'NRAPURACGD'   => 'NRAPURACGD',
                'CDCLIENTE'    => 'CDCLIENTE',
                'INICIAL'      => 'DTINICIAL',
                'FINAL'        => 'DTFINAL',
                'QUEBRAFILIAL' => 'IDQUEBRAFILIAL',
                'NRORG'        => 'NRORG'
            )
        ),
        'FAI01308' =>
        array (
            'strategy'          => 'QR2',
            'parameterMapping'  =>
            array (
                'FILIAL'        => 'P_CDFILIAL',
                'TFILIAL'       => 'P_TCDFILIAL',
                'NRAPURACGD'    => 'P_NRAPURACGD',
                'CLIENTE'       => 'P_CDCLIENTE',
                'TCLIENTE'      => 'P_TCDCLIENTE',
                'INICIAL'       => 'P_DTINICIAL',
                'FINAL'         => 'P_DTFINAL',
                'QUEBRAFILIAL'  => 'P_IDQUEBRAFILIAL'
            )
        )
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        try{
            $row = $request->getRow();
            if ($row["__report_name"] == 'FAI01308') {
                $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
            } else {
                $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
            }
            
            $this->callProcedureApurac($row);
            
            $row["CDFILIAL"] = $row["TFILIAL"] === "T"? $row["TFILIAL"]: $this->criarListaValores(implode($row["FILIAL"], '_'));
            
            if ($row["TCLIENTE"] !== "T") {
                $clientes = array();
                foreach($row["CLIENTE"] as $cliente){
                    array_push($clientes, $cliente["CDFILIAL"].$cliente["CDCLIENTE"]);
                }
            }
            
            $row["CDCLIENTE"] = $row["TCLIENTE"] === "T"? $row["TCLIENTE"]: $this->criarListaValores(implode($clientes, '_'));
            $row["NRORG"] = $this->environment->getNrOrgTrab();
            
        } catch (\Exception $e) {
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
        }
    }
    
    private function callProcedureApurac($row) {
        $procedureName = "APURAC_RELGESDIRETA";

        $nrapuracgd = $this->newCode->getCodeByTableName('RELAPURFATT', $this->environment->getNrOrgTrab(), 1 , 9);
        $row['NRAPURACGD'] = 'T'.$nrapuracgd;
        
        $filiais = $row["TFILIAL"] === "T"? $this->findFiliais(): $row["FILIAL"];
        
        foreach($filiais as $cdfilial) {
            $this->executeProcedure($procedureName, $cdfilial, $row);
        }
    }
    
    private function findFiliais() {
        $conditions = array(
            array(
                "columnName" => "CDOPERADOR",
                "operator" => "=",
                "value" => $this->environment->getCdOperador()
            )
        );
        
        $rows = $this->crudService->getFilteredDataSet('filioper', $conditions)->getRows();
        
        $filiais = array();
        
        foreach($rows as $row) {
            array_push($filiais, $row['CDFILIAL']);
        }
        
        return $filiais;
    }
    
    private function findClientes($filiais) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "IN",
                "value" => $filiais
            )
        );
        $rows = $this->crudService->getFilteredDataSet('cliente_clienfilial', $conditions)->getRows();
        
        $clientes = array();
        
        foreach($rows as $row) {
            array_push($clientes, $row['CDFILIAL'].$row['CDCLIENTE']);
        }
        
        return $clientes;
    }

    private function executeProcedure($procedureName, $cdfilial, $row){
        $procedure = new StoredProcedure($this->connection, $procedureName);
        
        $procedure->addParam(new Param('P_CDFILIAL',   Param::PARAM_INPUT, $cdfilial));
        $procedure->addParam(new Param('P_DATAINI',    Param::PARAM_INPUT, $row['INICIAL']));
        $procedure->addParam(new Param('P_DATAFIN',    Param::PARAM_INPUT, $row['FINAL']));
        $procedure->addParam(new Param('P_NRAPURACGD', Param::PARAM_INPUT, $row['NRAPURACGD']));
        $procedure->addParam(new Param('P_CDCLIENTE',  Param::PARAM_INPUT, ''));
        
        $procedure->execute();
    }
    
    //Criado no TCO: cria lista de valores recebendo string como parametro
    public function criarListaValores($valores){
        if(strlen($valores) <= 4000){
            return $valores;
        }else{
            $lista = explode('_',$valores);
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $key){
                    $this->insertLista($nrlista, $key);
                }
                return "_NRLISTA=".$nrlista;
            } catch(\Exception $e){
                throw $e;
            }
        }
    }
    
    //Criado no TCO: insere lista no banco
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista,
            "NRORG" => $this->environment->getNrorgTrab()
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
}