<?php
namespace Teknisa\FAI\Modules\Tabelas\Controllers;

use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class Fai04500ReajusteDeValorDeServico extends \Teknisa\GER\Controllers\TeknisaCrud{
	protected $dataSourceName;
	protected $parameterBagColumns;
	protected $dataSourceNameToSave;
	protected $environment;
	protected $connection;

	public function __construct($crudService, $parameterBag, $connection, $environment){
		$this->connection = $connection;
		$this->environment = $environment;
		parent::__construct($crudService, $parameterBag);
	}

	public function findServico($request, $response){
		$this->dataSourceName = 'fai04500_grupfili_filial_servico';
		$this->parameterBagColumns = array('CDFILIAL', 'CDGRUPFILI');

		parent::find($request, $response);
	}

	public function findCliente($request, $response){
		$this->dataSourceName = 'fai04500_grupfili_filial_servico_cliente';
		$this->parameterBagColumns = array('CDFILIAL', 'CDGRUPFILI', 'FILISERV');

		parent::find($request, $response);
	}

	public function saveOperation($request, $response){
		try{
			$row = $request->getRow();
			
			if(!empty($this->verifyVigenciaIsValid($row))){
				$msg = "DTVIGENCIAMENORDTPROC ";
				
				foreach($this->verifyVigenciaIsValid($row) as $filial){
					$msg .= "<br>".$filial['CDFILIAL']." - "
						.$filial['NMFILIAL']."<br>";
				}
				
				$response->setError(new Error($msg, 500));
			} else {
				$this->updateServclie($row);
				$this->insertHistoric($row);
			}
			
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	private function updateServclie ($row){
		$sql = "UPDATE SERVCLIE
					    SET
					        DTSERVFATU =:P_DTSERVFATU,
					        VRSERVFATU = ROUND(VRSERVFATU + ( (VRSERVFATU *:P_PERCAJUSTE) / 100),2)
					WHERE
					    CDFILIAL IN (
					        SELECT DISTINCT
					            F.CDFILIAL
					        FROM
					            FILIAL F
					            INNER JOIN FILIOPER FO ON FO.NRORG = F.NRORG
					                                      AND   FO.CDFILIAL = F.CDFILIAL
					            INNER JOIN FILGRUPFILI FG ON F.NRORG = FG.NRORG
					                                         AND   F.CDFILIAL = FG.CDFILIAL
					        WHERE
					            FG.NRORG =:P_NRORG
					            AND   FO.CDOPERADOR =:P_CDOPERADOR
					            AND   (
					                :P_CDGRUPFILI = FG.CDGRUPFILI
					                OR    :P_CDGRUPFILI = 'T'
					            )
					    )
					    AND   (
					        ( INSTR(:P_CDFILIAL,CDFILIAL) > 0 )
					        OR    (:P_CDFILIAL = 'T' )
					    )
					    AND   (
					        ( INSTR(:P_CDSERVFILI,CDSERVFILI) > 0 )
					        OR    (:P_CDSERVFILI = 'T' )
					    )
					    AND   (
					        ( INSTR(:P_CDCLIENTE,CDCLIENTE) > 0 )
					        OR    (:P_CDCLIENTE = 'T' )
					    )";

		$params = array(
			"P_DTSERVFATU" => $row['DATA'],
			"P_PERCAJUSTE" => $row['AJUSTE'],
			"P_CDFILIAL" => $row['CDFILIAL'],
			"P_CDSERVFILI" => $row['CDSERVFILI'],
			"P_CDGRUPFILI" => $row['CDGRUPFILI'],
			"P_CDCLIENTE" => $row['CDCLIENTE'],
			"P_CDOPERADOR" => $this->environment->getCdOperador(),
			"P_NRORG" => $this->environment->getNrOrg()
		);
		
		try{
			
			$query = $this->connection->prepare($sql);
			$query->execute($params);
			
		} catch (Exception $e){
			throw $e;
		}

	}

	private function insertHistoric($row){

		$sql = "INSERT INTO HISTSERVFATU
				 (CDFILIAL, CDSERVFILI, CDCLIENTE, DTSRVFATUHIS, VRSERVFATHIS,
				  VRFATUDOMHIS, VRFATUSEGHIS, VRFATUTERHIS, VRFATUQUAHIS,
				  VRFATUQUIHIS, VRFATUSEXHIS, VRFATUSABHIS, VRFATUFERHIS,
				  DTALTEFATHIS, CDOPERLOGADO)
				  (SELECT
				     CDFILIAL, CDSERVFILI, CDCLIENTE, DTSERVFATU, VRSERVFATU,
				     VRSERVFATDOM, VRSERVFATSEG, VRSERVFATTER, VRSERVFATQUA,
				     VRSERVFATQUI, VRSERVFATSEX, VRSERVFATSAB, VRSERVFATFER,
				     SYSDATE, :P_CDOPERADOR
				  FROM
				     SERVCLIE
				  WHERE
				     ((INSTR (:P_CDFILIAL, CDFILIAL) >0) OR (:P_CDFILIAL = 'T')) AND
				     ((INSTR(:P_CDSERVFILI, CDSERVFILI)>0)OR(:P_CDSERVFILI = 'T')) AND
				     ((INSTR(:P_CDCLIENTE, CDCLIENTE)>0)OR(:P_CDCLIENTE = 'T'))
				  )";


		$params = array(
			"P_CDOPERADOR" => $this->environment->getCdOperador(),
			"P_CDFILIAL" => $row['CDFILIAL'],
			"P_CDSERVFILI" => $row['CDSERVFILI'],
			"P_CDCLIENTE" => $row['CDCLIENTE']
		);
		
		try{
			
			$query = $this->connection->prepare($sql);
			$query->execute($params);
			
		} catch(Exception $e) {
			throw $e;
		}
	}
	
	private function verifyVigenciaIsValid($row){
		
		$sql = "SELECT
				    T.CDFILIAL,
				    T.DTPROC,
				    FI.NMFILIAL
				 FROM
				    (
				        SELECT
				            PF.CDFILIAL,
				            DECODE(PG.IDUTILDTPROC,'S',PF.DTPROCESSA,SYSDATE) DTPROC
				        FROM
				            PARAMGERAL PG,
				            PARAMFILIAL PF
				        WHERE
				            (
				                ( INSTR(:P_CDFILIAL,PF.CDFILIAL) > 0 )
				                OR   (:P_CDFILIAL = 'T' )
				            )
				    ) T,
				    FILIAL FI
				 WHERE
				    T.CDFILIAL = FI.CDFILIAL
				    AND   FI.CDFILIAL IN (
				        SELECT DISTINCT
				            F.CDFILIAL
				        FROM
				            FILIAL F
				            INNER JOIN FILIOPER FO ON FO.NRORG = F.NRORG
				                                      AND   FO.CDFILIAL = F.CDFILIAL
				            INNER JOIN FILGRUPFILI FG ON F.NRORG = FG.NRORG
				                                         AND   F.CDFILIAL = FG.CDFILIAL
				        WHERE
				            FG.NRORG =:P_NRORG
				            AND   FO.CDOPERADOR =:P_CDOPERADOR
				            AND   (
				                :P_CDGRUPFILI = FG.CDGRUPFILI
				                OR    :P_CDGRUPFILI = 'T'
				            )
				    ) AND
				T.DTPROC >:P_DATA ORDER BY T.CDFILIAL";
				   
		$params = array(
			"P_CDFILIAL" => $row['CDFILIAL'],
			"P_DATA" => $row['DATA'],
			"P_CDGRUPFILI" => $row['CDGRUPFILI'],
			"P_CDOPERADOR" => $this->environment->getCdOperador(),
			"P_NRORG" => $this->environment->getNrOrg()
		);
		
		try{
			$query = $this->connection->prepare($sql);
			$query->execute($params);
			
			return $query->fetchAll();
            
        } catch (Exception $e) {
            throw $e;
        }
	}
	
	public function verifyServico($request, $response){
		$row = $request->getRow();
		
		$sql = "SELECT
					    DTSRVFATUHIS
					FROM
					    HISTSERVFATU
					WHERE
					    CDFILIAL IN (
					        SELECT DISTINCT
					            F.CDFILIAL
					        FROM
					            FILIAL F
					            INNER JOIN FILIOPER FO ON FO.NRORG = F.NRORG
					                                      AND   FO.CDFILIAL = F.CDFILIAL
					            INNER JOIN FILGRUPFILI FG ON F.NRORG = FG.NRORG
					                                         AND   F.CDFILIAL = FG.CDFILIAL
					        WHERE
					            FG.NRORG =:P_NRORG
					            AND   FO.CDOPERADOR =:P_CDOPERADOR
					                AND   (
					                :P_CDGRUPFILI = FG.CDGRUPFILI
					                OR    :P_CDGRUPFILI = 'T'
					            )
					    ) AND
					( ( INSTR(:P_CDFILIAL,CDFILIAL) > 0 ) OR (:P_CDFILIAL = 'T' ) ) AND (
					    ( INSTR(:P_CDSERVFILI,CDSERVFILI) > 0 )
					    OR (:P_CDSERVFILI = 'T' )
					)
					                                                                AND (
					    ( INSTR(:P_CDCLIENTE,CDCLIENTE) > 0 )
					    OR (:P_CDCLIENTE = 'T' )
					)
					                                                                AND ( 
					    DTSRVFATUHIS >:P_DTSERVFATU 
					)";
				   
				   
		$params = array(
			"P_CDFILIAL" => $row['CDFILIAL'],
			"P_CDSERVFILI" => $row['CDSERVFILI'],
			"P_CDCLIENTE" => $row['CDCLIENTE'],
			"P_DTSERVFATU" => $row['DATA'],
			"P_CDGRUPFILI" => $row['CDGRUPFILI'],
			"P_CDOPERADOR" => $this->environment->getCdOperador(),
			"P_NRORG" => $this->environment->getNrOrg()
		);
		
		try{
			$query = $this->connection->prepare($sql);
			$query->execute($params);
			$retorno = $query->fetchAll();
			
			$response->addDataSet(new DataSet("DADOS", $retorno));
			
        } catch (Exception $e) {
            throw $e;
        }
	}
}